package net.knarcraft.permissionsigns.formatting;

import org.bukkit.configuration.file.YamlConfiguration;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;

/**
 * A tool to get strings translated to the correct language
 */
public class Translator {

    private static Map<TranslatableMessage, String> translatedMessages;
    private static Map<TranslatableMessage, String> backupTranslatedMessages;

    /**
     * Loads the languages used by this translator
     */
    public Translator() {
        backupTranslatedMessages = loadTranslatedMessages("en");
        translatedMessages = loadTranslatedMessages("en");
    }

    /**
     * Gets a translated version of the given translatable message
     *
     * @param translatableMessage <p>The message to translate</p>
     * @return <p>The translated message</p>
     */
    public static String getTranslatedMessage(TranslatableMessage translatableMessage) {
        if (translatedMessages.containsKey(translatableMessage)) {
            return translatedMessages.get(translatableMessage);
        } else if (backupTranslatedMessages.containsKey(translatableMessage)) {
            return backupTranslatedMessages.get(translatableMessage);
        } else {
            return translatableMessage.toString();
        }
    }

    /**
     * Loads all translated messages for the given language
     *
     * @param language <p>The language chosen by the user</p>
     * @return <p>A mapping of all strings for the given language</p>
     */
    public static Map<TranslatableMessage, String> loadTranslatedMessages(String language) {
        Map<TranslatableMessage, String> translatedMessages = new HashMap<>();
        InputStream inputStream = Translator.class.getResourceAsStream("/strings.yml");
        if (inputStream == null) {
            throw new IllegalArgumentException("Unable to load strings");
        }
        BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));
        YamlConfiguration configuration = YamlConfiguration.loadConfiguration(reader);

        for (TranslatableMessage message : TranslatableMessage.values()) {
            String translated = configuration.getString(language + "." + message.toString());
            if (translated != null) {
                translatedMessages.put(message, translated);
            }
        }
        return translatedMessages;
    }

}